/* Base64: This utility takes data from standard input and encodes it
 * into base64 ascii based encoding.
 */

/*Copyright (c) 2005, Aaron Hsu
 *All rights reserved.
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are
 *met:
 *
 ** Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 ** Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *
 ** Neither the name of the Wen-yao Animation & Programing nor the names
 *  of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written
 *  permission.
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* char get64char(char c):
 * a simple switch statement to return the equivalents for the base64 
 * alphabet.
 */
char get64char(char c)
{
	switch (c) {
    	case 0: return 'A';
    	case 1: return 'B';
    	case 2: return 'C';
    	case 3: return 'D';
    	case 4: return 'E';
    	case 5: return 'F';
  		case 6: return 'G';
  		case 7: return 'H';
  		case 8: return 'I';
  		case 9: return 'J';
		case 10: return 'K';
		case 11: return 'L';
		case 12: return 'M';
		case 13: return 'N';
		case 14: return 'O';
		case 15: return 'P';
		case 16: return 'Q';
		case 17: return 'R';
		case 18: return 'S';
		case 19: return 'T';
		case 20: return 'U';
		case 21: return 'V';
		case 22: return 'W';
		case 23: return 'X';
		case 24: return 'Y';
		case 25: return 'Z';
		case 26: return 'a';
		case 27: return 'b';
		case 28: return 'c';
		case 29: return 'd';
		case 30: return 'e';
		case 31: return 'f';
		case 32: return 'g';
		case 33: return 'h';
		case 34: return 'i';
		case 35: return 'j';
		case 36: return 'k';
		case 37: return 'l';
		case 38: return 'm';
		case 39: return 'n';
		case 40: return 'o';
		case 41: return 'p';
		case 42: return 'q';
		case 43: return 'r';
		case 44: return 's';
		case 45: return 't';
		case 46: return 'u';
		case 47: return 'v';
		case 48: return 'w';
		case 49: return 'x';
		case 50: return 'y';
		case 51: return 'z';
		case 52: return '0';
		case 53: return '1';
		case 54: return '2';
		case 54: return '3';
		case 55: return '4';
		case 56: return '5';
		case 57: return '6';
		case 58: return '7';
		case 59: return '8';
		case 60: return '9';
		case 61: return '0';
		case 62: return '+';		
		case 63: return '/';		
		case 64: return '=';
		default: return '\0';
	}
}

/* char *encode64(char *inp, char *out)
 * Takes a 3-char array and outputs a base64 4-char array representation 
 * of the 3-char array input.
 */
char *encode64(char *inp, char *out)
{
    char temp, temp2;
	int i = 0;
	
	/* Get first 6 bits on first char */
	temp = inp[0];
	temp >> 2;
	out[0] = get64char(temp);
	
	/* Get last 2 bits on first, first 4 bits on second */
	temp = inp[0];
	temp = temp & 0x03;
 	temp << 4;
	temp2 = inp[1];
	temp2 >> 4;
	out[1] = get64char(temp | temp2);
	
	/* Last 4 bits on second, first 2 bits on third */
	temp = inp[1];
	temp = temp & 0x0F;
	temp << 2;
	temp2 = inp[2];
	temp2 >> 6;
	out[2] = get64char(temp | temp2);
	
	/* Last 6 bits on third */
	temp = inp[2];
	temp = temp & 0x3F;
	out[3] = get64char(temp);
	
	return out;
}

#define TESTSTRING "This is a test."

int main()
{
	char string2[15];
	int i = 0;
	int i2 = 0;
	char string[] = TESTSTRING;

	/* Print base64 alphabet */
	for (i = 0; i < 65; i++) {
		printf("%c", get64char(i));
	}
	
	/* Process the string */
	/* Convert newlines */
	
	while (strlen(string) - i > 2) {
		/* Process group */
		encode64(string[i], string2[i2]);
		i += 3;
		i2 += 4;
	}
	
	/* Add final padding */
	switch (strlen(string) - i) {
		case 2:
			
}
